# Purpose:  A find and replace dialog singleton class
#
# Notes:	Don't know why I didn't implement this with messages.
#					Need to go back and do that.
#

require 'fox12'
require 'singleton'

require 'SessionManager'

module Mondrian

	SEARCH_BEGIN   = 0
	SEARCH_END     = 1
	SEARCH_RESULT  = 2
	
	class SearchFilesDialog < FXDialogBox
		include Notifier
		include Singleton
		
		def initialize
			super(SessionManager.instance.main_window, "Search Files", DECOR_TITLE | DECOR_BORDER | DECOR_CLOSE, 0, 0, 0, 0)
			
			frame = FXHorizontalFrame.new(self, FRAME_NONE)
			
			left_frame = FXVerticalFrame.new(frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
			project_frame = FXHorizontalFrame.new(left_frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
			FXLabel.new(project_frame, "Search in:")
			@project_field = FXListBox.new(project_frame, nil, 0, LAYOUT_RIGHT | FRAME_NORMAL | LISTBOX_NORMAL | LAYOUT_FIX_WIDTH)
			@project_field.setNumVisible(5)
			
			find_frame = FXHorizontalFrame.new(left_frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
			FXLabel.new(find_frame, "Fi&nd what:")
			@find_field = FXTextField.new(find_frame, 25, self, ID_ACCEPT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_RIGHT)
			@find_field.connect(SEL_CHANGED, method(:onFindTextChanged))
			
			@match_case = FXCheckButton.new(left_frame, "Match Case")

			right_frame = FXVerticalFrame.new(frame, FRAME_NONE, 0, 0, 0, 0, 10, 0, 0, 0)
			
			@find = FXButton.new(right_frame, "&Find", nil, self, ID_ACCEPT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@find.width = 75
			@find.disable

			@cancel = FXButton.new(right_frame, "&Cancel", nil, self, ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@cancel.width = 75
			
			self.connect(SEL_CONFIGURE) { @project_field.width = @find_field.width }
		end
		
		
		def onFindTextChanged(sender, message, data)
			if data == ""
				@find.disable
			else
				@find.enable
			end
		end


		def execute(default_project = nil)
			# populate the "search in" field
			@project_field.clearItems
			@project_field.appendItem("Entire Workspace")
			@project_field.appendItem("Non-project Files")
			SessionManager.instance.projects.each { |project|
				@project_field.appendItem(project.project_name)
				@project_field.setItemData(@project_field.numItems - 1, project)
			}
			@project_field.numVisible = @project_field.numItems
		
      @find_field.selectAll
			@find_field.setFocus
			
			if super(PLACEMENT_OWNER) == 1		
				results = valid_files = 0
	
				case @project_field.currentItem
					when 0
						files = FileManager.instance.get_files
					when 1
						files = FileManager.instance.get_files.select { |file| SessionManager.instance.get_associated_project(file) == nil }
					else
						files = @project_field.getItemData(@project_field.currentItem).get_files
				end
	
				notify(SEARCH_BEGIN, @find_field.text)
				
				if @match_case.checked?
					pattern = Regexp.new(Regexp.escape(@find_field.text))
				else
					pattern = Regexp.new(Regexp.escape(@find_field.text), Regexp::IGNORECASE)
				end

				files.each { |file|
					if file.kind_of?(RubyFile)
						valid_files += 1
						ln = 0
						
						file.code.each { |line|
							ln += 1
							if line =~ pattern
								notify(SEARCH_RESULT, [file.full_name, ln, line.chomp])
								results += 1
							end
						}
					end
				}
				
				notify(SEARCH_END, [results, valid_files])
			end
		end
	end



	class FindReplaceDialog < FXDialogBox
		FIND_DIALOG 		= 0
		REPLACE_DIALOG 	= 1
	
		include Singleton

    def initialize
      super(SessionManager.instance.main_window, "Find", DECOR_TITLE | DECOR_BORDER | DECOR_CLOSE, 0, 0, 0, 0)
			
      frame = FXHorizontalFrame.new(self, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			
      left_frame = FXVerticalFrame.new(frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
			find_frame = FXHorizontalFrame.new(left_frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			
      FXLabel.new(find_frame, "Fi&nd what:")
      @find_field = FXTextField.new(find_frame, 25, nil, 0, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_RIGHT)
			@find_field.connect(SEL_COMMAND, method(:onFindNext))
			@find_field.connect(SEL_CHANGED, method(:onFindTextChanged))

			@replace_frame = FXHorizontalFrame.new(left_frame, FRAME_NONE, 0, 0, 0, 0, 0, 0, 0, 0)
			
			FXLabel.new(@replace_frame, "Replace with:")
      @replace_field = FXTextField.new(@replace_frame, 25, nil, 0, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY)
			@replace_field.connect(SEL_COMMAND, method(:onFindNext))
			
			@match_case 				= FXCheckButton.new(left_frame, "Match &case")
			@regular_expression = FXCheckButton.new(left_frame, "Regular &expression")
			@wrap_around 				= FXCheckButton.new(left_frame, "Wrap aroun&d")
			@wrap_around.checkState = true
      
      right_frame = FXVerticalFrame.new(frame, FRAME_NONE, 0, 0, 0, 0, 10, 0, 0, 0)
      
      @find_next = FXButton.new(right_frame, "&Find Next", nil, owner, CMD_FIND_NEXT, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@find_next.disable
			@find_next.connect(SEL_COMMAND, method(:onFindNext))
      
			button_width = @find_next.font.getTextWidth("Find Previous") + 25
			
			@find_next.resize(button_width, 0)

      @find_previous = FXButton.new(right_frame, "Find &Previous", nil, owner, CMD_FIND_PREVIOUS, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@find_previous.resize(button_width, 0)
			@find_previous.disable
			@find_previous.connect(SEL_COMMAND, method(:onFindPrevious))
			
      @replace		 = FXButton.new(right_frame, "&Replace", nil, owner, CMD_REPLACE, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@replace.resize(button_width, 0)
			@replace.disable
			@replace.connect(SEL_COMMAND, method(:onReplace))

      @replace_all = FXButton.new(right_frame, "Replace &All", nil, owner, CMD_REPLACE_ALL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@replace_all.resize(button_width, 0)
			@replace_all.disable
			@replace_all.connect(SEL_COMMAND, method(:onReplaceAll))
			
      @cancel = FXButton.new(right_frame, "Cancel", nil, self, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
			@cancel.resize(button_width, 0)
			
			@slave 				= nil
			@last_search 	= ""
    end
		
		
		def onFindNext(sender, message, data)
			return if @find_field.text == ""
			
			@slave.find_next(@find_field.text) if @slave
			@last_search = @find_field.text
		end
		
		
		def onFindPrevious(sender, message, data)
			return if @find_field.text == ""
			
			@slave.find_previous(@find_field.text) if @slave
			@last_search = @find_field.text
		end
		
		
		def onReplace(sender, message, data)
			return if @find_field.text == ""
			
			@slave.replace(@find_field.text, @replace_field.text) if @slave
			@last_search = @find_field.text
		end
		
		
		def onReplaceAll(sender, message, data)
			return if @find_field.text == ""
			
			@slave.replace_all(@find_field.text, @replace_field.text) if @slave
			@last_search = @find_field.text
		end
		
		
		def onFindTextChanged(sender, message, data)
			if data == ""
				@find_next.disable
				@find_previous.disable
				@replace.disable
				@replace_all.disable
			else
				@find_next.enable
				@find_previous.enable
				@replace.enable
				@replace_all.enable
			end
		end
		
		
		def execute(slave, style = FIND_DIALOG, default_text = nil)
			@slave = slave
			
			@find_field.text = @last_search if @last_search
			@find_field.text = default_text.chomp if default_text
			
			onFindTextChanged(nil, 0, @find_field.text)
			
			if style == REPLACE_DIALOG
				@replace_frame.show
				@replace.show
				@replace_all.show
				self.title = "Replace"
				resize(getDefaultWidth, getDefaultHeight)
			else
				@replace_frame.hide
				@replace.hide
				@replace_all.hide
				@find_field.x = @replace_field.x
				self.title = "Find"
				resize(getDefaultWidth, getDefaultHeight)
			end
			
      @find_field.selectAll
			@find_field.setFocus
			
			super(PLACEMENT_OWNER)
			
			@slave = nil
		end
		
		
		def last_search
			@last_search
		end
		
		
		def match_case?
			@match_case.checked?
		end
		
		
		def regular_expression?
			@regular_expression.checked?
		end
		
		
		def wrap_around?
			@wrap_around.checked?
		end
	end
	
end