# Purpose:  Display a dialog for editing program preferences
#

require 'fox12'
require 'SessionManager'

include Fox


module Mondrian
	class GeneralFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			@web_browser = FileChooser.new(self, "Web browser (for help):", 20)
			@web_browser.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			FXHorizontalFrame.new(self) { |frame|
				FXLabel.new(frame, "Auto-save files:")
				@timed_save = FXListBox.new(frame, nil, 0, FRAME_LINE | LISTBOX_NORMAL | LAYOUT_FIX_WIDTH, 0, 0, 150, 0)
				@timed_save.setNumVisible 6
				@timed_save << "Never"
				@timed_save << "Every minute"
				@timed_save << "Every 5 minutes"
				@timed_save << "Every 10 minutes"
				@timed_save << "Every 30 minutes"
				@timed_save << "Every hour"
				@timed_save.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			}
			
			FXHorizontalFrame.new(self) { |frame|
				@save_on_tab_switch = FXCheckButton.new(frame, "Auto-save file when tab is switched")
				@save_on_tab_switch.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			}
			
			get_preferences
		end
		
		
		def get_preferences
			session = SessionManager.instance
			
			@web_browser.text = session.restore("Preferences/Web Browser") if session.restore("Preferences/Web Browser")
			
			@timed_save.currentItem = {0=>0, 1=>1, 5=>2, 10=>3, 30=>4, 60=>5}[session.restore("Preferences/Timed Save")] if session.restore("Preferences/Timed Save")
			@save_on_tab_switch.checkState = session.restore("Preferences/Save On Tab Switch")
		end
		
		
		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Web Browser", @web_browser.text)
			session.store("Preferences/Timed Save", {0=>0, 1=>1, 2=>5, 3=>10, 4=>30, 5=>60}[@timed_save.currentItem])
			session.store("Preferences/Save On Tab Switch", @save_on_tab_switch.checked?)
		end
	end



	class EditorFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)

			self.target = tgt
			
			@line_numbers = FXCheckButton.new(self, "Display line numbers")
			@line_numbers.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@eol_markers = FXCheckButton.new(self, "Display EOL markers")
			@eol_markers.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			f = FXHorizontalFrame.new(self)
			FXLabel.new(f, "Tab width:")
			@tab_width = FXTextField.new(f, 5, nil, 0, TEXTFIELD_INTEGER | FRAME_LINE)
			@tab_width.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@convert_tabs = FXCheckButton.new(self, "Convert tabs to spaces")
			@convert_tabs.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }

			@show_tracker = FXCheckButton.new(self, "Show change tracker")
			@show_tracker.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			get_preferences
		end
		

		def get_preferences
			session = SessionManager.instance

			@line_numbers.checkState = session.restore("Preferences/Line Numbers")
			@eol_markers.checkState = session.restore("Preferences/EOL Markers")
			@tab_width.text = session.restore("Preferences/Tab Width").to_s if session.restore("Preferences/Tab Width")
			@convert_tabs.checkState = session.restore("Preferences/Convert Tabs")
			@show_tracker.checkState = session.restore("Preferences/Show Change Tracker")
		end
		

		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Line Numbers", @line_numbers.checked?)
			print @line_numbers.checked?
			session.store("Preferences/EOL Markers", @eol_markers.checked?)
			session.store("Preferences/Convert Tabs", @convert_tabs.checked?)
			session.store("Preferences/Tab Width", @tab_width.text.to_i)
			session.store("Preferences/Show Change Tracker", @show_tracker.checked?)
		end
	end
			
			
			
	class AppearanceFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			@default_styles = FXRadioButton.new(self, "Use Mondrian's default styles")
			@default_styles.connect(SEL_COMMAND) { 
				@customize_styles.check = false
				refresh
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			@customize_styles = FXRadioButton.new(self, "Use custom styles")
			@customize_styles.connect(SEL_COMMAND) { 
				@default_styles.check = false
				refresh
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
	
			style_frame = FXHorizontalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X)

			FXLabel.new(style_frame, "Choose styles for:")

			@style_list = FXListBox.new(style_frame, nil, 0, LISTBOX_NORMAL | LAYOUT_FILL_X | FRAME_LINE)
			@style_list.setNumVisible 10
			@style_list.connect(SEL_COMMAND) { refresh }

			frame = FXHorizontalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X)
			
			fv = FXVerticalFrame.new(frame, FRAME_NONE)
			
			@bold 			= FXCheckButton.new(fv, "Bold")
			@bold.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).bold = @bold.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			@italic 		= FXCheckButton.new(fv, "Italic")
			@italic.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).italic = @italic.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			@underline 	= FXCheckButton.new(fv, "Underline")
			@underline.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).underline = @underline.checked?
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}

			fv2 = FXVerticalFrame.new(fv, FRAME_NONE)
			
			fh = FXHorizontalFrame.new(fv2, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			FXLabel.new(fh, "Text Color:", nil, LAYOUT_CENTER_Y)
			@text = FXColorWell.new(fh, 0, nil, 0, LAYOUT_RIGHT | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | COLORWELL_OPAQUEONLY, 0, 0, 30, 30)
			@text.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).foreground = sprintf("%02x%02x%02x", FXBLUEVAL(@text.rgba), FXGREENVAL(@text.rgba), FXREDVAL(@text.rgba)).hex
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fh = FXHorizontalFrame.new(fv2, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
			@fill_label = FXLabel.new(fh, "Fill Color:", nil, LAYOUT_CENTER_Y)
			@fill = FXColorWell.new(fh, 0, nil, 0, LAYOUT_RIGHT | LAYOUT_FIX_WIDTH | LAYOUT_FIX_HEIGHT | COLORWELL_OPAQUEONLY, 0, 0, 30, 30)
			@fill.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).background = sprintf("%02x%02x%02x", FXBLUEVAL(@fill.rgba), FXGREENVAL(@fill.rgba), FXREDVAL(@fill.rgba)).hex
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fv = FXVerticalFrame.new(frame, FRAME_NONE | LAYOUT_RIGHT | LAYOUT_FILL_X)
			
			fh = FXHorizontalFrame.new(fv, FRAME_NONE | LAYOUT_RIGHT)
			FXLabel.new(fh, "Font:")
			@fonts = FXListBox.new(fh, nil, 0, LISTBOX_NORMAL | FRAME_LINE | LAYOUT_FILL_X | LAYOUT_FIX_WIDTH, 0, 0, 150, 0)
			@fonts.setNumVisible 10
			FXFont.listFonts("").each { |desc|
				@fonts.appendItem(desc.face)
			}
			@fonts.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).font = @fonts.getItemText(@fonts.currentItem)
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			fh = FXHorizontalFrame.new(fv, FRAME_NONE | LAYOUT_RIGHT)
			FXLabel.new(fh, "Size:")
			@size 			= FXTextField.new(fh, 6, nil, 0, TEXTFIELD_REAL | FRAME_LINE)
			@size.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			@size.connect(SEL_COMMAND) {
				@style_list.getItemData(@style_list.currentItem).size = @size.text.to_i
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			get_preferences
		end


		def get_preferences
			session = SessionManager.instance

			@styles = EditorStyles.new(session.restore("Preferences/Editor Styles"))
			
			@style_list.clearItems
			@style_list.appendItem("Normal text", nil, @styles.styles[STYLE_IDENTIFIER])
			@style_list.appendItem("Module names", nil, @styles.styles[STYLE_MODULE_NAME])
			@style_list.appendItem("Class names", nil, @styles.styles[STYLE_CLASS_NAME])
			@style_list.appendItem("Method names", nil, @styles.styles[STYLE_METHOD_NAME])
			@style_list.appendItem("Operators", nil, @styles.styles[STYLE_OPERATOR])
			@style_list.appendItem("Keywords", nil, @styles.styles[STYLE_KEYWORD])
			@style_list.appendItem("Numbers", nil, @styles.styles[STYLE_NUMBER])
			@style_list.appendItem("Strings", nil, @styles.styles[STYLE_UNTERMINATED_STRING])
			@style_list.appendItem("Shell commands", nil, @styles.styles[STYLE_UNTERMINATED_SYSTEM_COMMAND])
			@style_list.appendItem("Regular expressions", nil, @styles.styles[STYLE_UNTERMINATED_REGEXP])
			@style_list.appendItem("Array literals", nil, @styles.styles[STYLE_UNTERMINATED_ARRAY_LITERAL])
			@style_list.appendItem("Comments", nil, @styles.styles[STYLE_COMMENT])
			@style_list.appendItem("System Variables", nil, @styles.styles[STYLE_SYSTEM_VARIABLE])
			
			@default_styles.checkState = (not SessionManager.instance.restore("Preferences/Use Custom Styles"))
			@customize_styles.checkState = SessionManager.instance.restore("Preferences/Use Custom Styles")
			
			refresh
		end
		
		
		def refresh
			if @default_styles.checked?
				@style_list.disable
				@bold.disable
				@italic.disable
				@underline.disable
				@fill.disable
				@text.disable
				@fonts.disable
				@size.disable
			else
				@style_list.enable
				@bold.enable
				@italic.enable
				@underline.enable
				@fill.enable
				@text.enable
				@fonts.enable
				@size.enable
			end

			@bold.checkState = @style_list.getItemData(@style_list.currentItem).bold
			@italic.checkState = @style_list.getItemData(@style_list.currentItem).italic
			@underline.checkState = @style_list.getItemData(@style_list.currentItem).underline
			if @fonts.findItem(@style_list.getItemData(@style_list.currentItem).font) >= 0
				@fonts.currentItem = @fonts.findItem(@style_list.getItemData(@style_list.currentItem).font)
			end
			@size.text = @style_list.getItemData(@style_list.currentItem).size.to_s
			
			color = sprintf("%06x", @style_list.getItemData(@style_list.currentItem).foreground)
			@text.rgba = FXRGB(color[4..5].hex, color[2..3].hex, color[0..1].hex)
			
			color = sprintf("%06x", @style_list.getItemData(@style_list.currentItem).background)
			@fill.rgba = FXRGB(color[4..5].hex, color[2..3].hex, color[0..1].hex)
			
			if @style_list.getItemData(@style_list.currentItem).eol_filled
				@fill_label.show 
				@fill.show
			else
				@fill_label.hide 
				@fill.hide
			end
		end
		
		
		def apply
			session = SessionManager.instance
			
			[ STYLE_STRING,
				STYLE_REGEXP,
				STYLE_ARRAY_LITERAL,
				STYLE_SYSTEM_COMMAND ].each { |style|
					@styles.styles[style].font = @styles.styles[style + 1].font
					@styles.styles[style].bold = @styles.styles[style + 1].bold
					@styles.styles[style].size = @styles.styles[style + 1].size
					@styles.styles[style].encoding = @styles.styles[style + 1].encoding
					@styles.styles[style].underline = @styles.styles[style + 1].underline
					@styles.styles[style].italic = @styles.styles[style + 1].italic
					@styles.styles[style].foreground = @styles.styles[style + 1].foreground
			}
			
			@styles.styles[STYLE_WHITESPACE].font = @styles.styles[STYLE_IDENTIFIER].font
			@styles.styles[STYLE_WHITESPACE].bold = @styles.styles[STYLE_IDENTIFIER].bold
			@styles.styles[STYLE_WHITESPACE].size = @styles.styles[STYLE_IDENTIFIER].size
			@styles.styles[STYLE_WHITESPACE].encoding = @styles.styles[STYLE_IDENTIFIER].encoding
			@styles.styles[STYLE_WHITESPACE].underline = @styles.styles[STYLE_IDENTIFIER].underline
			@styles.styles[STYLE_WHITESPACE].italic = @styles.styles[STYLE_IDENTIFIER].italic
			@styles.styles[STYLE_WHITESPACE].foreground = @styles.styles[STYLE_IDENTIFIER].foreground
					
			session.store("Preferences/Use Custom Styles", @customize_styles.checked?)
			session.store("Preferences/Editor Styles", EditorStyles.new(@styles))
		end
	end



	class ExecutionFrame < FXGroupBox
		def initialize(parent, title, options, tgt)
			super(parent, title, options)
			
			self.target = tgt
			
			@ruby_path = FileChooser.new(self, "Ruby path:", 25)
			@ruby_path.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@auto_save = FXCheckButton.new(self, "Save files automatically when run")
			@auto_save.connect(SEL_COMMAND) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
			
			@capture_stdout = FXRadioButton.new(self, "Capture program stdout")
			@capture_stdout.connect(SEL_COMMAND) { 
				@run_terminal.check = false
				@terminal.disable if @terminal; 
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			@run_terminal = FXRadioButton.new(self, "Run program in a terminal window")
			@run_terminal.connect(SEL_COMMAND) { 
				@capture_stdout.check = false
				@terminal.enable if @terminal; 
				target.handle(self, MKUINT(0, SEL_CHANGED), nil) 
			}
			
			unless RUBY_PLATFORM =~ /win32/
				f = FXHorizontalFrame.new(self)
				FXLabel.new(f, "Terminal string:")
				@terminal = FXTextField.new(f, 25)
				@terminal.connect(SEL_CHANGED) { target.handle(self, MKUINT(0, SEL_CHANGED), nil) }
				@terminal.disable if @capture_stdout.checked?
			end
			
			get_preferences
		end


		def get_preferences
			session = SessionManager.instance
			
			@ruby_path.text = session.restore("Preferences/Ruby Path")

			@auto_save.checkState = session.restore("Preferences/Auto Save")
			@capture_stdout.checkState = session.restore("Preferences/Capture Stdout")
			@run_terminal.checkState = (not session.restore("Preferences/Capture Stdout"))

			unless RUBY_PLATFORM =~ /win32/
				@terminal.text = session.restore("Preferences/Terminal String") if session.restore("Preferences/Terminal String")
			end
		end
		
		
		def apply
			session = SessionManager.instance
			
			session.store("Preferences/Ruby Path", @ruby_path.text)
			
			session.store("Preferences/Auto Save", @auto_save.checked?)
			
			session.store("Preferences/Capture Stdout", @capture_stdout.checked?)
			session.store("Preferences/Terminal String", @terminal.text) if @terminal
		end
	end


	
	#
	# PreferenceDialog:	the main preferences window
	# 
	# Notes:	calls functions directly in the owner...bad idea, should fix at some point
	#
	
	class PreferencesDialog < FXDialogBox
    def initialize(owner)
      super(owner, "Edit Preferences", DECOR_TITLE | DECOR_BORDER, 0, 0, 0, 0)
			
			@target = owner
			
			session = SessionManager.instance
			
			@need_apply = false
      
			main_frame = FXVerticalFrame.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)
			
			content_frame = FXHorizontalFrame.new(main_frame)

			@shutter = FXShutter.new(content_frame, nil, 0, FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)
    
			FXShutterItem.new(@shutter, "  General  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Editor  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Appearance  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			FXShutterItem.new(@shutter, "  Execution  ", nil, 0, LAYOUT_FILL_X | LAYOUT_TOP | LAYOUT_LEFT)
			
			switcher = FXSwitcher.new(content_frame, FRAME_NONE | LAYOUT_FIX_HEIGHT | LAYOUT_FIX_WIDTH, 0, 0, 350, 250, 0, 0, 0, 0)

			@shutter.connect(SEL_COMMAND) { |sender, message, data| switcher.current = data }
			
			@general = GeneralFrame.new(switcher, "General Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@general.connect(SEL_CHANGED) { need_apply }
			@editor = EditorFrame.new(switcher, "Editor Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@editor.connect(SEL_CHANGED) { need_apply }
			@appearance = AppearanceFrame.new(switcher, "Appearance Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@appearance.connect(SEL_CHANGED) { need_apply }
			@execution = ExecutionFrame.new(switcher, "Execution Options", GROUPBOX_NORMAL | FRAME_SUNKEN | LAYOUT_FILL_X | LAYOUT_FILL_Y, self)
			@execution.connect(SEL_CHANGED) { need_apply }

			# OK/cancel buttons
			
			fh = FXHorizontalFrame.new(main_frame, FRAME_NONE | LAYOUT_FILL_X)
			
			button = FXButton.new(fh, "Restore Defaults")
			button.connect(SEL_COMMAND) {
				@target.set_default_preferences(true)
				
				@general.get_preferences
				@editor.get_preferences
				@appearance.get_preferences
				@execution.get_preferences
				
				need_apply				
				apply_settings
			}
			
      frame = FXHorizontalFrame.new(fh, FRAME_NONE | LAYOUT_RIGHT, 0, 0, 0, 0, 0, 0, 0, 0)
      
      button = FXButton.new(frame, "OK", nil, nil, 0, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(button.font.getTextWidth("Cancel") + 25, 0)
			button.connect(SEL_COMMAND) {
				handle(self, MKUINT(FXDialogBox::ID_ACCEPT, SEL_COMMAND), nil) if apply_settings
			}
      
      @apply_button = FXButton.new(frame, "Apply", nil, nil, 0, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      @apply_button.resize(@apply_button.font.getTextWidth("Cancel") + 25, 0)
			@apply_button.connect(SEL_COMMAND) { apply_settings }
			@apply_button.disable
			
      button = FXButton.new(frame, "Cancel", nil, self, FXDialogBox::ID_CANCEL, BUTTON_NORMAL | LAYOUT_FIX_WIDTH)
      button.resize(button.font.getTextWidth("Cancel") + 25, 0)
    end
		
		
		def need_apply
			@need_apply = true
			
			@apply_button.enable
		end
		
		
		def apply_settings
			return true unless @need_apply
			
			@general.apply
			@editor.apply
			@appearance.apply
			@execution.apply
			
			SessionManager.instance.save

			@target.apply_preferences
			
			@need_apply = false

			@apply_button.disable

			true
		end
  end
  
end