# Purpose:  provide a dockable frame for displaying results of
#						the search project dialog
#
# Notes:	largely copied from ExecutionMonitor.rb.  the scintilla-
#					as-list functionality could be factored out.
#

require 'fox12'
require 'fox12/scintilla'
require 'FileManager'
require 'DockFrame'
require 'FindReplaceDialog.rb'
require 'SessionManager'

include Fox
include Mondrian::Managers

module Mondrian

  class SearchFrame < Dockable
		attr_accessor :slave

		STYLE_MESSAGE						= 1
		STYLE_RESULT						= 2
		STYLE_SELECTED_RESULT		= 3

    def initialize(parent)
      super(parent)

			frame = FXHorizontalFrame.new(self, LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_THICK, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0)

      @scintilla = FXScintilla.new(frame, nil, 0, LAYOUT_FILL_X | LAYOUT_FILL_Y | SCROLLERS_NORMAL)
      
			if RUBY_PLATFORM =~ /win32/
				@scintilla.setEOLMode(FXScintilla::SC_EOL_CRLF)
			else
				@scintilla.setEOLMode(FXScintilla::SC_EOL_LF)
			end

      @scintilla.setReadOnly(true)
      @scintilla.setEndAtLastLine(false) # fixes a scrolling bug in the widget
      
      @scintilla.styleSetSize(FXScintilla::STYLE_DEFAULT, 12) unless RUBY_PLATFORM =~ /win32/

			# need to make a global setting for this
			if RUBY_PLATFORM =~ /win32/
				fixed_font 			= "Courier New"
				fixed_font_size = 8
			else
				fixed_font 			= "fixed"
				fixed_font_size = 12
			end
			
			@scintilla.styleSetFore(STYLE_MESSAGE, 					0x000000)
			@scintilla.styleSetFore(STYLE_RESULT, 					0x000000)
      @scintilla.styleSetFont(STYLE_RESULT,  					fixed_font)
			@scintilla.styleSetSize(STYLE_RESULT,						fixed_font_size)
			@scintilla.styleSetFore(STYLE_SELECTED_RESULT, 	0xffffff)
			@scintilla.styleSetBack(STYLE_SELECTED_RESULT, 	0xff0000)
      @scintilla.styleSetFont(STYLE_SELECTED_RESULT, 	fixed_font)
			@scintilla.styleSetSize(STYLE_SELECTED_RESULT,	fixed_font_size)
			
			@scintilla.connect(SEL_COMMAND) { |sender, message, data|
				if data.nmhdr.code == FXScintilla::SCN_DOUBLECLICK
					view_result(@scintilla.lineFromPosition(@scintilla.getCurrentPos) + 1)
				end
			}
			
			self.title = "Search Results"

			@scintilla.setScrollWidth(900) # try and stop the horizontal scrollbar from appearing except when really needed
			
			SearchFilesDialog.instance.add_observer(self)
			
			@results = {}
			@current_result = nil
      
			@slave = nil
    end


		def append_styled_text(text, style)
      return unless text
			
			result = String.new(text)
      
			result.size.times { |i|
				result[i * 2 + 1, 0] = style.chr
			}
			
      @scintilla.setReadOnly(false)
      
			@scintilla.documentEnd
			@scintilla.addStyledText(result.length, result)

      @scintilla.setReadOnly(true)
		end


		def view_result(line)
			return unless @results[line] and @slave
			
			result = @results[line]

			begin
				file = FileManager.instance.open_file(result[0]) # try to open the file in the current working directory
			rescue
				FXMessageBox.error(SessionManager.instance.main_window, MBOX_OK, "View Search Result", $!.message)
				return
			end
			
			if @current_result
				@scintilla.startStyling(@scintilla.positionFromLine(@current_result - 1), 0xff)
				@scintilla.setStyling(@scintilla.getLineEndPosition(@current_result - 1) - @scintilla.positionFromLine(@current_result - 1), STYLE_RESULT)
			end
			
			@scintilla.startStyling(@scintilla.positionFromLine(line - 1), 0xff)
			@scintilla.setStyling(@scintilla.getLineEndPosition(line - 1) - @scintilla.positionFromLine(line - 1), STYLE_SELECTED_RESULT)

			@scintilla.gotoLine(line - 1)

			viewer = @slave.launch_viewer(nil, file) if file
			
			viewer.goto_line(result[1]) if viewer

			@current_result = line
		end


		def reset
      @scintilla.setReadOnly(false)
			@scintilla.clearAll
      @scintilla.setReadOnly(true)	
		end


    def update(sender, message, data)
			case message
				when SEARCH_BEGIN
					reset
					request_show
					append_styled_text("Searching for '#{data}'...\n", STYLE_MESSAGE)
					
				when SEARCH_RESULT
					append_styled_text("#{data[0]} (#{data[1]}): #{data[2]}", STYLE_RESULT)
					append_styled_text("\n", STYLE_RESULT)
					@results[@scintilla.getLineCount - 1] = data
					
				when SEARCH_END
					append_styled_text("Search complete: #{data[0]} match" + (data[0] == 1 ? "" : "es") + " found in #{data[1]} file" + (data[1] == 1 ? "." : "s."), STYLE_MESSAGE)
			end
    end
  end

end